#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Program that allows to download all at once the 'PROCESSED_DATA' from synchrotron
for a given beamline (several dates possible) and a given acronym.
Login information can be provided either in command line or in interactive mode.
"""

# ----------------------- Imports --------------------------------
import sys
from pathlib import Path
import datetime
import time
import argparse
import paramiko

# my modules:
import config
from modules.utils import verbose
from modules.subcommands import get_esrf_data


# ----------------------- Functions-------------


def parse_arguments():
    """Parser"""
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description='''
    ---------------------------------------------------------
    Program that allows to download all at once the 'PROCESSED_DATA' from synchrotron 
    for a given beamline (several dates possible) and a given acronym.
    Login information can be provided either in command line or in interactive mode.
    Intended to make ligand and fragment screening easy!
    ---------------------------------------------------------''')
    parser.add_argument("-s", "--synchrotron", default="ESRF", choices=['ESRF'], help="Which synchrotron?")
    parser.add_argument("-l", "--login", help="synchrotron login (Example: 'mx----')")
    parser.add_argument("-w", "--password", help="password for connexion at synchrotron")
    parser.add_argument("-b", "--beamline", default="", help="name of the beamline")
    parser.add_argument("-d", "--dates", default="",
                        help="date of the run (format AAAAMMJJ) or list of dates for the same run "
                             "(format 'AAAAMMJJ AAAAMMJJ')")
    parser.add_argument("-p", "--protein", default="", help="protein acronym")
    parser.add_argument("-g", "--group", default="",
                        help="group added via MxCube3 as a supplementary folder level before acronym")
    return parser.parse_args()


#----------------------Main program------------------------

if __name__ == "__main__":

    """Analyse of parser"""
    args = parse_arguments()
    print("easyget will try to retrieve data from synchrotron: ", args.synchrotron)
    verbose.question_y_n("Shall we go on ?")

    """Paramiko"""
    paramiko.util.log_to_file('paramiko.log')

    """User informations"""
    login = args.login  # login
    mypassword = args.password  # password
    #beam = args.beamline  # nom de la ligne
    #dates = args.dates.split()  # liste de dates sous forme YYYYMMDD
    #acronym = args.protein  # nom de la prot

    """Values depending on synchrotron"""
    if args.synchrotron == 'ESRF':
        server = config.ESRF_SERVER
        port = config.ESRF_PORT


    """Test of sftp connexion"""
    paths_ok = []  # liste des chemins des jeux à récupérer

    if not args.login:
        while True:
            login = input("Login? (Example: 'mx----') : ")
            if len(login) == 6:
                break
    if not mypassword:
        while True:
            mypassword = input("Password? : ")
            break

    try:
        transport = paramiko.Transport((server, port))
        transport.connect(username=login, password=mypassword)
        sftp_client = paramiko.SFTPClient.from_transport(transport)
    except paramiko.AuthenticationException as e:
        print("\n" + str(
            e) + "\nProblem in the connection, check that the following parameters are correct: login and password. Or try again later ...")
        sys.exit(0)

    beam, dates, group, acronym, paths_ok = get_esrf_data.test_path_proc_data(sftp_client, login, args)

    start_date = datetime.datetime.now().strftime("%Y-%m-%d_%H%M%S")

    """Creates a download folder"""
    work_dir = Path.cwd()
    data_dir = work_dir.joinpath('PROCESSED_DATA')
    data_dir.mkdir(exist_ok=True)

    """Creates a log file with information on download"""
    nom_proc_log = "download-proc-data_" + args.synchrotron + "_" + beam + "_" + "_".join(
        dates) + "_" + acronym + "_" + start_date + ".log"
    proc_log = verbose.log_start(work_dir, 'logs_easyget', nom_proc_log, verbose.easyget_logo)
    verbose.print_and_write("Login: " + login, proc_log)
    verbose.print_and_write("Password: " + mypassword, proc_log)
    verbose.print_and_write("Beamline name: " + beam, proc_log)
    verbose.print_and_write("Date of the run (format YYYYMMDD): " + ", ".join(dates), proc_log)
    if group:
        verbose.print_and_write("Group: " + group, proc_log)
    verbose.print_and_write("Acronym: " + acronym, proc_log)
    verbose.print_and_write(verbose.chaine_xfois(config.motif, config.nb_motifs), proc_log)

    """Listing of data to download in log"""
    jeux_proc, recup_data = get_esrf_data.list_proc_data(paths_ok, sftp_client, acronym, dates, proc_log)


    """Real beginning"""
    start = time.process_time()
    start_perf = time.perf_counter()

    """Download the processing subfolders (with only the files in the 'results' folders) for each autoprocessing"""
    get_esrf_data.download_results(sftp_client, data_dir, jeux_proc, recup_data, proc_log)

    sftp_client.close()
    transport.close()
    verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), proc_log)
    verbose.print_and_write("\nDone ...\n", proc_log)


    """Written at the end of each log file"""
    # performance
    verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs, "Program performance"), proc_log)
    end = time.process_time()
    end_perf = time.perf_counter()
    totalCPU = end - start
    total_perf = end_perf - start_perf
    #verbose.print_and_write("Duration / CPU (hours:minutes:seconds) = " + str(datetime.timedelta(seconds=totalCPU)), log)
    #verbose.print_and_write("Duration / real time (hours:minutes:seconds) = " + str(datetime.timedelta(seconds=total_perf)) + "\n", log)
    #verbose.print_and_write("Duration / CPU = " + "{0} hours : {1} minutes : {2} seconds"
    #      .format(*verbose.seconds_to_hhmmss(int(totalCPU))) + ("(" + str(totalCPU) + " seconds)" if totalCPU < 1 else ""), log)
    verbose.print_and_write("Duration = " + "{0} hours : {1} minutes : {2} seconds"
          .format(*verbose.seconds_to_hhmmss(int(total_perf))) + ("(" + str(total_perf) + " seconds)\n" if total_perf < 1 else "\n"), proc_log)






