# -*- coding: utf-8 -*-

"""
Get ESRF data from 'results' folders in each process folder of each dataset.
"""

#--------------------------Imports-----------------------------
import os
from pathlib import Path
from pathlib import PurePosixPath
import sys
import collections
import stat

# My modules
from modules.utils import verbose
from modules.utils import sftp
import config

#-------------------------------------------------------
beamlines = config.ESRF_BEAMLINES
data_path = config.ESRF_PATH

#----------------------- Functions-------------
def sftp_get_results(sftp_client, path_res, proc_dir_local, log):
    """
    Download remotely ESRF result data in 'results' folder.
    Copy of the content of 'results' folder (which contains mtz) => only files, into the target folder in proc_dir_local
    :param sftp_client: sftp client connection with paramiko
    :param path_res: contains the results folder of the files to be copied
    :param proc_dir_local: folder where to copy processes by creating a folder (target folder) with the name of the process.
    :param log: log file with path
    """
    proc = Path(path_res).name  # nom du dossier sans le path
#    print("Process directory name to create if mtz in results directory: " + proc)
    try:
        sftp_client.chdir(path_res)
        sftp_client.chdir('results')
        test_mtz = [f for f in sftp_client.listdir(sftp_client.getcwd()) if ".mtz" in f]
        if test_mtz:   # s'il y a des fichier mtz dans le dossier results
#            print("\nlist of files in results directory with mtz:")
#            for f in test_mtz:
#                print(f)
            Path(proc_dir_local).joinpath(proc).mkdir(exist_ok=True)
            verbose.print_and_write("=> New directory created: " + proc, log)
            os.chdir(str(Path(proc_dir_local).joinpath(proc)))
            # gestion des liens symboliques dans le dossier 'results' comme dans le cas de parallel_proc (avec S_ISLNK)
            for f in sftp_client.listdir_attr(sftp_client.getcwd()):
                if stat.S_ISREG(f.st_mode) or stat.S_ISLNK(f.st_mode): #check if is file
                    verbose.print_and_write("File to copy: " + str(PurePosixPath(path_res).joinpath('results', f.filename)), log)
                    verbose.print_and_write("File to be copied: " + str(Path(proc_dir_local).joinpath(proc, f.filename)), log)
                    sftp_client.get(str(PurePosixPath(path_res).joinpath('results', f.filename)), str(Path(proc_dir_local).joinpath(proc, f.filename)))
        else:
            verbose.print_and_write("No mtz files in this results directory ...", log)
    except:
        pass
    sftp_client.chdir(path_res)
    os.chdir(proc_dir_local)


def test_path_proc_data(sftp_client, login, args):
    """
    Tests if path to processed data is correct
    :param sftp_client: sftp client connection with paramiko
    :param login: login to connect to ESRF
    :param args: args from parser
    :return: beam, dates, acronym, and the list of paths of datasets to retrieve
    """
    beam = args.beamline  # nom de la ligne
    dates = args.dates.split()  # liste de dates sous forme YYYYMMDD
    acronym = args.protein  # nom de la prot
    group = args.group  # nom d'un éventuel niveau supplémentaire (group dans MxCube)
    paths_ok = []  # liste des paths des jeux à récupérer
    if not beam or beam not in beamlines:
        while True:
            beam = input(
                "Beamline name (" + ",".join(beamlines) + ")? : ").lower()  # met le nom en minuscules
            if beam in beamlines:
                break
    source_proc_data = str(PurePosixPath(data_path).joinpath(login, beam))
    try:
        # navigation sur la source
        sftp_client.chdir(source_proc_data)
    except FileNotFoundError as e:
        print(str(e))
        print("\nProblem in the connection, check that the following parameter is correct: beamline name.")
        sys.exit(0)
    if not dates:
        while True:
            dates = input(
                "Date of the run? (format AAAAMMJJ) or list of dates for the same run "
                "(format AAAAMMJJ AAAAMMJJ) ? : ").split()
            dates_ok = True
            for da in dates:
                if len(da) != 8:
                    dates_ok = False
            if dates_ok:
                break
    for da in dates:  # parcourir la liste des dates pour un même run
        # vérifier le chemin vers "PROCESSED_DATA"
        try:
            source_proc_data = str(PurePosixPath(data_path).joinpath(login, beam, da, "PROCESSED_DATA"))
            # navigation sur la source
            sftp_client.chdir(source_proc_data)
        except FileNotFoundError as e:
            print(str(e))
            print(
                "\nProblem in the connection, check that the following parameter is correct: date = " + da + ".")
            sys.exit(0)
    if not acronym:
        while True:
            acronym = input("Acronym?: ")
            break
    if not group:
        group = input("Is there a group (put in MxCube) as a parent folder for data? (if not, just press enter): ")
    for da in dates:  # parcourir la liste des dates pour un même run
        # gère le cas où un niveau de dossier supplémentaire (= groupe dans MxCube3)
        try:
            source_proc_data = str(PurePosixPath(data_path).joinpath(login, beam, da, "PROCESSED_DATA", group, acronym))
            # navigation sur la source
            sftp_client.chdir(source_proc_data)
        except FileNotFoundError as e:
            print(str(e))
            print(
                "\nProblem in the connection, check that the following parameter is correct: acronym = " + acronym +
                (" and group = " + str(group) if group else "") + " for date = " + da + ".")
            sys.exit(0)
        else:
            paths_ok.append(source_proc_data)
    return beam, dates, group, acronym, paths_ok



def list_proc_data(paths_ok, sftp_client, acronym, dates, proc_log):
    """
    Lists the data to be download and ask if the program should go on.
    :param paths_ok: list of paths of datasets to retrieve
    :param sftp_client: sftp_client connection with paramiko
    :param acronym: name of protein acronym
    :param dates: list of dates for ESRF runs
    :param proc_log: log file with path
    :return: jeux_proc = list of tuples (dataset, autoprocessing folder)
             recup_data = list of tuples (date, path, list of datasets OK, list of datasets without autoprocessing,
                            dictionary of the number of processes per dataset)
    """


    """List of the data to be retrieved"""
    recup_data = []  # liste de tuples (date, path, [jeux OK], [jeux sans autoprocessing], {dictionnaire du nombre de process par jeu})
    jeux_proc = []  # liste de tuples (jeu, dossier autoprocessing)
    paths_ok.sort()

    for path in paths_ok:
        jeux = []  # liste des jeux à copier pour chaque path
        jeux_ok = []  # liste des jeux avec autoprocessing
        jeux_without_proc = []  # liste des jeux sans autoprocessing
        sftp_client.chdir(path)
        """List of datasets"""
        # on vérifie que c'est un dossier et que l'acronyme est dedans
        jeux = [j.filename for j in sftp_client.listdir_attr() if stat.S_ISDIR(j.st_mode) and acronym in j.filename]
        jeux.sort()
        verbose.print_and_write("\nList of folders found in " + path + " (" + str(len(jeux)) + " folders) : ", proc_log)
        for j in jeux:
            verbose.print_and_write(j, proc_log)

        """List of autoprocessing folders for each dataset"""
        for jeu in jeux:
            sftp_client.chdir(jeu)
            for l in sftp_client.listdir_iter():
                if stat.S_ISDIR(l.st_mode) and (
                        l.filename.startswith("autoprocessing_") and not l.filename.startswith("autoprocessing_ref")):
                    jeux_proc.append([jeu, l.filename])
            sftp_client.chdir(path)
        jeux_proc.sort(key=lambda t: (t[0], t[
            1].lower()))  # trier la liste de tuples selon le premier élément, puis le deuxième (casse indépendant => 'A' avant 'e')

        """calculation of the number of autoprocessing per dataset"""
        nbproc_par_jeu = collections.Counter(
            elem[0] for elem in jeux_proc)  # Counter => pas forcément dans l'ordre des élements
        sorted(nbproc_par_jeu.items())  # trier par item, cad par jeu
        verbose.print_and_write("\nNumber of 'processing' in each folder : ", proc_log)
        for j in jeux:
            verbose.print_and_write(j + ", nb proc : " + str(nbproc_par_jeu[j]), proc_log)
            if nbproc_par_jeu[j] != 0:  # count of a missing element is zero
                jeux_ok.append(j)
            else:
                jeux_without_proc.append(j)

        sorted(jeux_ok)
        sorted(jeux_without_proc)
        for date in dates:
            if date in path:
                recup_data.append((date, path, jeux_ok, jeux_without_proc, nbproc_par_jeu))
                break
    verbose.print_and_write(
        "\nList of datasets that will NOT be downloaded (" + str(sum(len(r[3]) for r in recup_data)) + ") :", proc_log)
    for date, path, j_OK, j_without_proc, nbp_par_j in recup_data:
        verbose.print_and_write("-for " + date + " (" + str(len(j_without_proc)) + ") : ", proc_log)
        for j in j_without_proc:
            verbose.print_and_write(j, proc_log)
    verbose.print_and_write("\nList of datasets that will be downloaded (" + str(sum(len(r[2]) for r in recup_data)) + ") :",
                    proc_log)
    for date, path, j_OK, j_without_proc, nbp_par_j in recup_data:
        verbose.print_and_write("-for " + date + " (" + str(len(j_OK)) + ") : ", proc_log)
        for j in j_OK:
            verbose.print_and_write(j, proc_log)
    verbose.print_and_write("\n", proc_log)

    verbose.question_y_n("Shall we go on ?")
    return jeux_proc, recup_data


def download_results(sftp_client, data_dir, jeux_proc, recup_data, proc_log):
    """
    Download the processing subfolders (with only the files in the 'results' folders) for each autoprocessing
    :param sftp_client: sftp_client connection with paramiko
    :param data_dir: folder where data will be downloaded
    :param jeux_proc: list of tuples (dataset, autoprocessing folder)
    :param recup_data: list of tuples (date, path, list of datasets OK, list of datasets without autoprocessing,
                            dictionary of the number of processes per dataset)
    :param proc_log: log file with path
    """
    print("\nFiles downloaded:")
    verbose.print_and_write("Data sets :", proc_log)
    for date, chemin, j_OK, j_without_proc, nbp_par_j in recup_data:
        sftp_client.chdir(chemin)
        for jeu in j_OK:
            # tester si le dossier jeu existe (cas où collecté sur 2 dates),
            # ou ne pas copier si dossier existe !! TODO: à tester
            if not data_dir.joinpath(jeu).exists():
                data_dir.joinpath(jeu).mkdir(
                    exist_ok=True)  # If exist_ok is true, FileExistsError exceptions will be ignored
                sftp_client.chdir(jeu)
                verbose.print_and_write("\n- " + jeu, proc_log)
                nb = 0  # nb d'autoprocessing
                for (j, p) in jeux_proc:
                    if j == jeu and nb in range(nbp_par_j[jeu]):
                        sftp_client.chdir(p)
                        nb += 1
                        data_dir.joinpath(jeu, "PROC_" + str(nb)).mkdir(exist_ok=True)
                        verbose.print_and_write("\nNew directory: " + "PROC_" + str(nb), proc_log)
                        # lister les dossiers contenant les sous-dossiers 'results'
                        list_path_with_res = [path for (path, dos, f) in
                                              sftp.sftp_walk(sftp_client, sftp_client.getcwd())
                                              if 'results' in dos]
                        verbose.print_and_write("\nList of process directories: ", proc_log)
                        for path_result in list_path_with_res:
                            #                    sftp_client.chdir(path_result)
                            verbose.print_and_write("\n" + path_result, proc_log)
                            sftp_get_results(sftp_client, path_result, str(data_dir.joinpath(jeu, "PROC_" + str(nb))),
                                             proc_log)
                        sftp_client.chdir(str(PurePosixPath(chemin).joinpath(jeu)))
                sftp_client.chdir(chemin)
            else:
                verbose.print_and_write("!!!!!!!!!!!!!!!!!!! The folder " + jeu + " already exists, "
                                                                                  "it has not been copied.", proc_log)











