# -*- coding: utf-8 -*-

"""'reindex' subcommand """

# ----------------------- Imports --------------------------------
import sys
import os
from pathlib import Path
import datetime
import subprocess
import csv
import shutil

# My modules
import config
from modules.utils import verbose
from modules.utils import xtriage
from modules.utils import proteins

#--------------------------Functions------------------------------
# For Pointless
if os.name == 'posix':
    shell_choice = False
elif os.name == 'nt':
    shell_choice = True


def reindex_pointless(mtz, ref_file, sg_ref, new_mtz_file, pointless_logname):
    """
    Try to reindex a mtz file according to a mtz reference file. If it fails, try with a reference space group.
    Write a new mtz file in the same folder as the one of the original mtz file.
    :param mtz: mtz file to reindex
    :param ref_file: reference mtz file
    :param sg_ref: space group of reference
    :param new_mtz_file: mtz file created by Pointless
    :param pointless_logname: pointless log file name with path
    :return: log message
    """
    message = ""
    # pointless with reference file:
    # TODO: ajouter text=True à la place de encoding='utf-8' quand python >= 3.7
    pointless = subprocess.Popen([config.POINTLESS_EXEC, "hklref", ref_file, "hklin", mtz, "hklout", new_mtz_file],
                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell_choice, encoding='utf-8')
    out, err = pointless.communicate()
    # Ecrire le fichier log de pointless dans le dossier
    with open(pointless_logname, 'w') as pointless_log:
        pointless_log.write(out)
        pointless_log.write(err)
    # TESTER si "ERROR" dans le log et relancer avec SPACEGROUP
    pointless_err = False
    with open(pointless_logname, 'r') as pointless_log:
        lignes = pointless_log.readlines()
        for l in lignes:
            if "**** ERROR ****" in l:
                pointless_err = True
                break
    if  pointless_err:
        pointless_logname_sg = str(Path(pointless_logname).parent.joinpath(Path(pointless_logname).stem + '_sg.log'))
        pointless_input = "spacegroup " + str(proteins.find_key(proteins.sg_nb_to_symbol, sg_ref))  # des fois bug avec sg (ex : I222)
        # TODO: ajouter text=True à la place de encoding='utf-8' quand python >= 3.7
        pointless = subprocess.Popen([config.POINTLESS_EXEC, "hklin", mtz, "hklout", new_mtz_file],
                                     stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell_choice, encoding='utf-8')
        out, err = pointless.communicate(input=pointless_input.encode())
        with open(pointless_logname_sg, 'w') as pointless_log:
            pointless_log.write(out)
            pointless_log.write(err)
        message = "Pointless launched with space group ..."
        with open(pointless_logname_sg, 'r') as pointless_log:
            lignes = pointless_log.readlines()
            for l in lignes:
                if "**** ERROR ****" in l:
                    message += "\n!!!!!!!!!!!!!! ERROR in pointless either with reference file or space group, " \
                               "see pointless log file ..."
    else:
        message = "Pointless completed with reference file ..."
    return message



def reindex_run(data_dir, ref_mtz):
    """
    'reindex' subcommand,
    'prep' subcommand should have been launched before
    :param data_dir: data directory
    :param ref_mtz: reference mtz (with path)
    :return: log file
    """

    date = datetime.datetime.now().strftime("%Y-%m-%d_%H%M%S")

    """Folder where we work"""
    work_dir = Path.cwd()
    """The existence of following path (created with 'prep' subcommand) has been checked in easypipe"""
    phenix_dir = work_dir.joinpath('easYPipe')
    prep_dir = phenix_dir.joinpath('1a_prep')
    mtz_treat_csv = prep_dir.joinpath('mtz_to_treat_ALL.csv')


    """Determination of the space group of mtz reference file"""
    sg_mtzref = ""
    reindex_dir_ref = phenix_dir.joinpath('1b_reindex_SG_ref-mtz')
    reindex_dir_ref.mkdir(exist_ok=True)
    # xtriage log pour mtz ref
    xtriage_mtz_log = reindex_dir_ref.joinpath("xtriage_" + ref_mtz.stem + ".log")
    xtriage_mtz_verbose = reindex_dir_ref.joinpath("xtriage-verbose_" + ref_mtz.stem + ".log")
    if not xtriage_mtz_verbose.exists():
        message_Xtriage = "Xtriage launched for: " + ref_mtz.name
        xtriage.xtriage_mtz(ref_mtz, xtriage_mtz_verbose, xtriage_mtz_log)
    else:
        message_Xtriage = "Xtriage already launched for: " + ref_mtz.name
    if xtriage_mtz_log.exists():
        sg_mtzref = xtriage.sg_in_xtriage_log(xtriage_mtz_log)
    else:
        sys.exit("Space group not found in ref mtz")

    """Creation of reindex directory with space group information"""
    reindex_dir = phenix_dir.joinpath('1b_reindex_' + sg_mtzref)
    reindex_dir.mkdir(exist_ok=True)

    """Create a log file with reindexation informations"""
    reindex_dir.joinpath('logs').mkdir(exist_ok=True)
    nom_reindex_log = "reindex_easypipe_" + date + ".log"
    reindex_log = verbose.log_start(reindex_dir, 'logs', nom_reindex_log, verbose.easypipe_logo)
    verbose.print_and_write("Reference mtz file: " + str(ref_mtz), reindex_log)
    verbose.print_and_write("Determination of the space group of reference mtz file (cf folder: "
                                  + reindex_dir_ref.name + ")", reindex_log)
    verbose.print_and_write(message_Xtriage, reindex_log)
    verbose.print_and_write("Space group of reference mtz file: " + sg_mtzref, reindex_log)
    verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), reindex_log)
    verbose.print_and_write("Creation of the directory: " + reindex_dir.name, reindex_log)
    verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), reindex_log)

    """Read mtz information in csv file and copy mtz to reindex in reindex folder"""
    mtz_copy_list = []  # liste des mtz copiés pour traitement, sg correspondant
    verbose.print_and_write(
        verbose.cadre_text(config.motif, config.nb_motifs, "Listing of mtz file to reindex"), reindex_log)
    verbose.print_and_write("Search for information in csv file: 'mtz_to_treat_ALL.csv'", reindex_log)
    verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs), reindex_log)
    verbose.print_and_write(
        "Copy of mtz files to reindex (space group different from reference) in '" + reindex_dir.name +
        "' folder:\n", reindex_log)
    with open(mtz_treat_csv, newline='') as csvfile:
        mtz_csv_reader = csv.DictReader(csvfile)
        for row in mtz_csv_reader:
            if row['space group'] != sg_mtzref:
                # copie le mtz à traiter dans le dossier reindex, en gardant l'arborescence dataset/mtznb_PROC
                mtz_to_copy = phenix_dir.joinpath('0_processed_datasets', row['dataset'], 'data', row['mtz nb'] + '_'
                                                  + row['process name'], row['mtz file'])
                mtz_copy = reindex_dir.joinpath(row['dataset'], row['mtz nb'] + '_' + row['process name'],
                                                row['mtz file'])
                if not mtz_copy.exists():
                    verbose.print_and_write('/'.join(mtz_copy.parts[len(mtz_copy.parts) - 3:]) + " ... copied",
                                                  reindex_log)
                    reindex_dir.joinpath(row['dataset']).mkdir(exist_ok=True)
                    reindex_dir.joinpath(row['dataset'], row['mtz nb'] + '_' + row['process name']).mkdir(exist_ok=True)
                    shutil.copyfile(str(mtz_to_copy), str(mtz_copy))
                else:
                    verbose.print_and_write(
                        '/'.join(mtz_copy.parts[len(mtz_copy.parts) - 3:]) + " ... already copied!", reindex_log)
                mtz_copy_list.append([mtz_copy, row['space group']])

    """Reindexing"""
    mtz_reindex = []  # liste des mtz réindexé, attention si échec le fichier n'existe pas
    verbose.print_and_write(
        verbose.cadre_text(config.motif, config.nb_motifs, "Reindexing mtz files with Pointless"),
        reindex_log)
    for mtz, sg in mtz_copy_list:
        message_pointless = "Pointless not launched yet ..."
        os.chdir(str(mtz.parent))  # on va dans le dossier contenant le mtz
        verbose.print_and_write("Pointless launched for: " + "/".join(mtz.parts[-3:]), reindex_log)
        verbose.print_and_write("Original space group: " + sg, reindex_log)
        new_mtz_file = mtz.parent.joinpath(
            mtz.stem + "_reindexed_" + sg_mtzref + ".mtz")  # nom du fichier mtz ré-indexé
        mtz_reindex.append(new_mtz_file)
        pointless_logname = mtz.parent.joinpath(mtz.stem + "_pointless_" + sg_mtzref + ".log")
        message_pointless = reindex_pointless(str(mtz), ref_mtz, sg_mtzref, str(new_mtz_file), str(pointless_logname))
        verbose.print_and_write(message_pointless, reindex_log)
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                reindex_log)

    """Launch Xtriage on reindexed mtz file to get informations"""
    mtz_reindex_OK = []  # liste que les fichiers qui existent i.e. pour lesquels Pointless est OK
    verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs,
                                                           "Getting informations from reindexed mtz files with Xtriage"),
                                  reindex_log)
    for mtz in mtz_reindex:
        if mtz.exists():
            mtz_reindex_OK.append(mtz)
            nom_xtriage = mtz.parent.joinpath("xtriage_" + mtz.stem + ".log")
            xtriage_verbose = mtz.parent.joinpath("xtriage-verbose.log")
            if not xtriage_verbose.exists():
                verbose.print_and_write("Xtriage launched for: " + mtz.name, reindex_log)
                xtriage.xtriage_mtz(mtz, xtriage_verbose, nom_xtriage)
            else:
                verbose.print_and_write("Xtriage already launched for: " + mtz.name, reindex_log)

    """Modification of mtz_to_treat_ALL.csv file and copy of reindexed mtz in processed_datasets/prep"""
    # ne le modifie que si nécessaire
    if mtz_reindex_OK:
        verbose.print_and_write(
            verbose.cadre_text(config.motif, config.nb_motifs, "Modification of mtz_to_treat_ALL.csv file "
                                                                            "and copy of reindexed mtz "), reindex_log)
        # Renommage du csv d'origine pour sauvegarde
        mtz_treat_csv_ori = reindex_dir.joinpath(mtz_treat_csv.stem + "_before-reindex.csv")
        verbose.print_and_write("Changing name of csv file before reindexing: " + mtz_treat_csv_ori.name,
                                      reindex_log)
        verbose.print_and_write("Moving csv file before reindexing to '" + reindex_dir.name + "' folder",
                                      reindex_log)
        os.rename(str(mtz_treat_csv), str(mtz_treat_csv_ori))
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                reindex_log)
        # Création d'un nouveau csv modifié (nom du mtz et complétude)
        verbose.print_and_write(
            "Creating new mtz_to_treat_ALL.csv file ... will be stored in " + prep_dir.name + " folder", reindex_log)
        verbose.print_and_write("Updating mtz file name and completeness when necessary:\n", reindex_log)
        with open(mtz_treat_csv_ori, newline='') as csvfile_ori:
            mtz_csv_reader = csv.DictReader(csvfile_ori)
            header = mtz_csv_reader.fieldnames  # récupère le header
            with open(mtz_treat_csv, 'w', newline='') as csvfile:
                csv_writer = csv.writer(csvfile, quoting=csv.QUOTE_MINIMAL)
                csv_writer.writerow(header)  # header
                for row in mtz_csv_reader:
                    mtz_diff = False  # est-ce que le mtz a été réindexé i.e est différent de l'origine
                    for mtz in mtz_reindex_OK:
                        # chercher le mtz à remplacer en comparant le nom du dossier parent
                        if mtz.parts[-3] == row['dataset'] and mtz.parts[-2] == row['mtz nb'] + "_" + row[
                            'process name']:
                            for fi in mtz.parent.iterdir():
                                # Récupération des infos dans le log Xtriage
                                if fi.is_file and fi.name.startswith("xtriage_"):
                                    completeness = xtriage.completeness_in_xtriage_log(fi)
                                    space_group = xtriage.sg_in_xtriage_log(fi)
                            verbose.print_and_write("Dataset: " + row['dataset'], reindex_log)
                            verbose.print_and_write("New mtz for " + row['mtz nb'] + ": " + mtz.name, reindex_log)
                            verbose.print_and_write("New completeness: " + str(completeness) + "%", reindex_log)
                            verbose.print_and_write("Original space group: " + row['space group'], reindex_log)
                            verbose.print_and_write("New space group: " + str(space_group), reindex_log)
                            csv_writer.writerow(list(row.values())[0:3] + [mtz.name] + [list(row.values())[4]]
                                                + [completeness] + [space_group] + list(row.values())[7:])
                            # copie du mtz réindexé dans 'processed_data'
                            verbose.print_and_write("Copy of new mtz in 'processed_data'", reindex_log)
                            mtz_new = phenix_dir.joinpath('0_processed_datasets', row['dataset'], 'data',
                                                          row['mtz nb'] + "_" + row['process name'], mtz.name)
                            shutil.copyfile(mtz, mtz_new)
                            # nom de l'ancien mtz changé en .old
                            verbose.print_and_write("Changing name of original mtz .old\n", reindex_log)
                            mtz_old = Path(str(mtz_new).split('_reindexed_')[0] + ".mtz")
                            os.rename(mtz_old, str(mtz_old) + ".old")
                            mtz_diff = True
                            break
                    if not mtz_diff:
                        csv_writer.writerow(list(row.values()))
    else:
        verbose.print_and_write(verbose.chaine_xfois(config.sub_motif, config.nb_motifs),
                                reindex_log)
        verbose.print_and_write("No mtz reindexed successfully ...", reindex_log)

    """Summary"""
    verbose.print_and_write(verbose.chaine_xfois(config.motif, config.nb_motifs), reindex_log)
    verbose.print_and_write(verbose.easypipe_logo, reindex_log)
    verbose.print_and_write(verbose.cadre_text(config.motif, config.nb_motifs, "SUMMARY "),
                            reindex_log)
    verbose.print_and_write("mtz candidates for reindexing (space group different from reference file): "
                                  + str(len(mtz_copy_list)) + "\n", reindex_log)
    for mtz, sg in mtz_copy_list:
        verbose.print_and_write("/".join(mtz.parts[-3:]) + " / " + sg, reindex_log)
    verbose.print_and_write("\nNumber of reindexed mtz: " + str(len(mtz_reindex_OK)), reindex_log)
    if mtz_reindex_OK:
        for mtz in mtz_reindex_OK:
            verbose.print_and_write("/".join(mtz.parts[-3:]), reindex_log)
        verbose.print_and_write(
            "\nYou can visualize new 'mtz_to_treat_ALL.csv' file after reindexing in '1a_prep' folder, "
            "and previous one in '" + reindex_dir.name + "' folder.", reindex_log)

    return reindex_log

